<?php

namespace Drupal\mautic_paragraph_commerce\EventSubscriber;

use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\mautic_paragraph_commerce\MauticCommerceApiInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Queue\QueueFactory;
use Drupal\state_machine\Event\WorkflowTransitionEvent;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Psr\Log\LoggerInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpFoundation\RequestStack;
use Drupal\Core\Locale\CountryManager;

/**
 * Connect commerce order with Mautic.
 */
class OrderMauticSubscriber implements EventSubscriberInterface {

  use StringTranslationTrait;

  /**
   * The logger service.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The client IP address.
   *
   * @var string
   */
  protected $clientIp;

  /**
   * The host address.
   *
   * @var string
   */
  protected $host;

  /**
   * The configuration.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $config;

  /**
   * The commerce_wishlist_expiration queue.
   *
   * @var \Drupal\Core\Queue\QueueInterface
   */
  protected $queue;

  /**
   * The MauticCommerceApi service.
   *
   * @var \Drupal\mautic_paragraph_commerce\MauticCommerceApiInterface
   */
  protected $mauticCommerceApi;

  /**
   * Constructs a new Event Subscriber object.
   *
   * @param \Psr\Log\LoggerInterface $logger
   *   A logger instance.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   The corresponding request stack.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The configuration factory.
   * @param \Drupal\Core\Queue\QueueFactory $queue_factory
   *   The queue factory.
   * @param \Drupal\mautic_paragraph_commerce\MauticCommerceApiInterface $mautic_paragraph_commerce_api
   *   The Mautic API service.
   */
  public function __construct(LoggerInterface $logger, EntityTypeManagerInterface $entity_type_manager, RequestStack $request_stack, ConfigFactoryInterface $config_factory, QueueFactory $queue_factory, MauticCommerceApiInterface $mautic_paragraph_commerce_api) {
    $this->logger = $logger;
    $this->entityTypeManager = $entity_type_manager;
    $this->clientIp = $request_stack->getCurrentRequest()->getClientIp();
    $this->host = $request_stack->getCurrentRequest()->getSchemeAndHttpHost();
    $this->config = $config_factory;
    $this->mauticCommerceApi = $mautic_paragraph_commerce_api;
    $this->queue = $queue_factory->get('mautic_paragraph_commerce_add_contact_to_segment');
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    // The format for adding a state machine event to subscribe to is:
    // {group}.{transition key}.pre_transition or
    // {group}.{transition key}.post_transition
    // depending on when you want to react.
    $events = [
      'commerce_order.place.post_transition' => [
        'addCustomerToMauticSegment',
        100,
      ],
    ];
    return $events;
  }

  /**
   * Add customer to Mautic segments based on order items and promotions.
   *
   * @param \Drupal\state_machine\Event\WorkflowTransitionEvent $event
   *   The transition event.
   *
   * @throws \GuzzleHttp\Exception\GuzzleException
   */
  public function addCustomerToMauticSegment(WorkflowTransitionEvent $event) {
    $mautic_config = $this->config->get('mautic_paragraph_commerce.settings');
    if ($mautic_config->get('mautic_segment_disable') == TRUE) {
      return;
    }

    // Get the order entity.
    /** @var \Drupal\commerce_order\Entity\OrderInterface $order */
    $order = $event->getEntity();

    $segmet_list = $this->getMauticSegmentList($order);

    if ($mautic_config->get('mautic_segment_debug') == TRUE) {
      $this->logger->debug('Marketing Mautic segmnet list: <pre>@data</pre>', [
        '@data' => print_r($segmet_list, TRUE),
      ]);
    }

    if (!empty($segmet_list)) {
      // Get customer details.
      $email = $order->getEmail();
      /** @var \Drupal\profile\Entity\ProfileInterface $billing_profile */
      $billing_profile = $order->getBillingProfile();
      if ($billing_profile && $billing_profile->address) {
        $address = $billing_profile->address->first();
        $first_name = $address->getGivenName();
        $last_name = $address->getFamilyName();
        $country_code = $address->getCountryCode();
        $city = $address->getLocality();
      }

      // Process country name.
      $countries = CountryManager::getStandardList();
      $full_country_name = $countries[$country_code]->__toString();

      // Get tags.
      $tags_list = $this->getMauticTags($order);

      $data = [
        'firstname' => $first_name ?? '',
        'lastname' => $last_name ?? '',
        'country' => $full_country_name ?? '',
        'city' => $city ?? '',
        'email' => $email,
        'website' => $this->host,
        'tags' => $tags_list,
        'ipAddress' => $this->clientIp,
        'overwriteWithBlank' => TRUE,
      ];
      $queue_data['data'] = $data;
      $queue_data['segment_list'] = $segmet_list;

      if ($mautic_config->get('mautic_segment_debug') == TRUE) {
        $this->logger->debug('Contact data before Mautic contact creation: <pre>@data</pre>', [
          '@data' => print_r($queue_data['data'], TRUE),
        ]);
        $this->logger->debug('Final Mautic segment list to add contact to: <pre>@data</pre>', [
          '@data' => print_r($queue_data['segment_list'], TRUE),
        ]);
      }

      if ($mautic_config->get('mautic_segment_queue') == TRUE) {
        // Create queue item to be processed on next cron.
        $this->queue->createItem($queue_data);
      }
      else {
        $this->addCustomerToSegment($data, $segmet_list);
      }
    }

  }

  /**
   * Create and add customer to Mautic segment.
   *
   * @param array $data
   *   The customer data.
   * @param array $segment_list
   *   The segment list.
   */
  private function addCustomerToSegment(array $data, array $segment_list) {
    // Create Mautic contact from customer billing profile.
    $mautic_contact = $this->mauticCommerceApi->createContact($data);

    // Add contact to segments.
    foreach ($segment_list as $segment_id) {
      if (isset($mautic_contact['contact']['id'])) {
        if ($mautic_contact['contact']['id']) {
          $response = $this->mauticCommerceApi->addContactToSegment($segment_id, $mautic_contact['contact']['id']);
          if (isset($response['errors'])) {
            if ($response['errors']) {
              // Get error message.
              $error_message = explode('Response:', $response['errors']['0']['message'])[0];
              $this->logger->warning('Could not add contact to segment with id %segment_id. %error',
                ['%segment_id' => $segment_id, '%error' => $error_message]);
            }
          }
        }
      }
    }
  }

  /**
   * Set product title as Mautic tag.
   *
   * @param \Drupal\commerce_order\Entity\OrderInterface $order
   *   The customer order.
   *
   * @return array
   *   The list of tags.
   */
  private function getMauticTags(OrderInterface $order) {

    $tags = [];

    /** @var \Drupal\commerce_order\Entity\OrderItemInterface $order_item */
    foreach ($order->getItems() as $order_item) {

      // Get product variation.
      /** @var \Drupal\commerce_product\Entity\ProductInterface $product */
      $product = $order_item->getPurchasedEntity()->getProduct();
      $tags[] = $product->label();
    }
    return $tags;
  }

  /**
   * Generate list of Mautic tags.
   *
   * @param \Drupal\commerce_order\Entity\OrderInterface $order
   *   The customer order.
   *
   * @return array
   *   The list of tags.
   */
  private function getMauticSegmentList(OrderInterface $order) {

    $marketing_terms = $order->getData('mautic_terms_and_conditions');
    $marketing_list = [];
    $exclude_from_marketing_list = [];
    $applied_promotions = [];

    // Get list of promotions applied to this order.
    foreach ($order->collectAdjustments() as $adjustment) {
      if ($adjustment->getType() === 'promotion') {
        $applied_promotions[$adjustment->getSourceId()] = $adjustment->getLabel();
      }
    }

    // List enabled mautic commerce conditions.
    $mautic_commerce_conditions = $this->entityTypeManager->getStorage('mautic_commerce_condition')
      ->loadByProperties(['status' => 1]);
    foreach ($mautic_commerce_conditions as $mautic_condition) {
      /** @var \Drupal\mautic_paragraph_commerce\MauticCommerceConditionInterface $mautic_condition */
      $mautic_cond_settings = $mautic_condition->getSettings();

      // Process mautic commerce conditions.
      if (!empty($mautic_cond_settings['commerce_promotions'])) {
        foreach ($mautic_cond_settings['commerce_promotions'] as $promotion) {
          if (array_key_exists($promotion, $applied_promotions)) {
            if ($mautic_condition->process($order)) {
              if ($mautic_cond_settings['exclude_from_marketing_terms'] == TRUE) {
                $exclude_from_marketing_list[$mautic_cond_settings['mautic_segment']] = $mautic_cond_settings['mautic_segment'];
              }
              else {
                $marketing_list[$mautic_cond_settings['mautic_segment']] = $mautic_cond_settings['mautic_segment'];
              }
            }
          }
        }
      }
      else {
        if ($mautic_condition->process($order)) {
          if ($mautic_cond_settings['exclude_from_marketing_terms'] == TRUE) {
            $exclude_from_marketing_list[$mautic_cond_settings['mautic_segment']] = $mautic_cond_settings['mautic_segment'];
          }
          else {
            $marketing_list[$mautic_cond_settings['mautic_segment']] = $mautic_cond_settings['mautic_segment'];
          }
        }
      }
    }

    // Merge segment lists.
    if ($marketing_terms == TRUE) {
      $final_segment_list = array_unique(array_merge($marketing_list, $exclude_from_marketing_list), SORT_REGULAR);
    }
    else {
      $final_segment_list = $exclude_from_marketing_list;
    }
    return $final_segment_list;
  }

}
