<?php

namespace Drupal\mautic_paragraph_commerce\Form;

use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\mautic_paragraph_commerce\MauticCommerceApiInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Mautic Commerce Condition form.
 *
 * @property \Drupal\mautic_paragraph_commerce\MauticCommerceConditionInterface
 *   $entity
 */
class MauticCommerceConditionForm extends EntityForm {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The messenger to send info or warnings to Drupal with.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The product storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $promotionStorage;

  /**
   * The module handler service.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * The MauticParagraphApi service.
   *
   * @var \Drupal\mautic_paragraph_commerce\MauticCommerceApiInterface
   */
  protected $mauticCommerceApi;

  /**
   * Constructs a new Mautic commerce condition object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger to send info or warnings to Drupal with.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler service.
   * @param \Drupal\mautic_paragraph_commerce\MauticCommerceApiInterface $mautic_paragraph_commerce_api
   *   The Mautic API service.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, MessengerInterface $messenger, ModuleHandlerInterface $module_handler, MauticCommerceApiInterface $mautic_paragraph_commerce_api) {
    $this->entityTypeManager = $entity_type_manager;
    $this->messenger = $messenger;
    $this->promotionStorage = $entity_type_manager
      ->getStorage('commerce_promotion');
    $this->moduleHandler = $module_handler;
    $this->mauticCommerceApi = $mautic_paragraph_commerce_api;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('messenger'),
      $container->get('module_handler'),
      $container->get('mautic_paragraph_commerce_api')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {

    $form = parent::form($form, $form_state);

    $segments_list = [];
    $disable_element = FALSE;
    try {
      if ($this->mauticCommerceApi->getStatus()) {
        $segments_list = $this->mauticCommerceApi->getSegmentList();
      }
    }
    catch (\Exception $e) {
      watchdog_exception('error', $e);
      $this->messenger->addError($this->t('Failed to retrieve Mautic segments. there was a problem with the connection to the Mautic API instance.'));
      $form['connector_config']['status'] = [
        '#type' => 'fieldset',
        '#title' => $this->t('Status'),
        '#title_display' => 'before',
        '#open' => TRUE,
        '#description' => $this->t('Connection failed, there was a problem with the connection to the Mautic API instance.'),
      ];
      $disable_element = TRUE;
    }

    $segments = [];
    foreach ($segments_list as $item) {
      $segments[$item['id']] = $item['name'];
    }

    $segment_id = $form_state->getValue('mautic_segment');
    if (isset($segment_id)) {
      $this->entity->setLabel($segments[$segment_id]);
    }

    $form['label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Label'),
      '#maxlength' => 255,
      '#default_value' => $this->entity->label(),
      '#description' => $this->t('Label for the mautic commerce condition.'),
      '#required' => FALSE,
      '#disabled' => TRUE,
      '#access' => FALSE,
    ];

    if ($this->entity->isNew()) {
      $form['id'] = [
        '#type' => 'machine_name',
        '#default_value' => uniqid('mcc_', FALSE),
        '#machine_name' => [
          'exists' => '\Drupal\mautic_paragraph_commerce\Entity\MauticCommerceCondition::load',
        ],
        '#disabled' => TRUE,
        '#access' => FALSE,
      ];
    }

    $form['description'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Description'),
      '#default_value' => $this->entity->get('description'),
      '#description' => $this->t('Description of the mautic commerce condition.'),
    ];

    $form['mautic_settings'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Settings'),
    ];

    $form['mautic_settings']['mautic_segment'] = [
      '#type' => 'select2',
      '#title' => $this->t('Mautic segment'),
      '#default_value' => $this->entity->getSettings('mautic_segment') ?? [],
      '#options' => $segments,
      '#empty_option' => $this->t('Select the Segment'),
      '#required' => TRUE,
      '#ajax' => [
        'callback' => '::ajaxRefresh',
      ],
      '#disabled' => $disable_element,
    ];

    $form['mautic_settings']['commerce_promotions'] = [
      '#title' => $this->t('Commerce Promotions'),
      '#type' => 'select2',
      '#description' => $this->t('You can enter or paste multiple entries separated by comma or semicolon'),
      '#default_value' => $this->entity->getSettings('commerce_promotions') ?? [],
      '#multiple' => TRUE,
      '#tags' => TRUE,
      '#autocomplete' => TRUE,
      '#selection_handler' => 'default',
      '#target_type' => 'commerce_promotion',
      '#required' => FALSE,
      '#validated' => TRUE,
    ];

    if ($this->moduleHandler->moduleExists('commerce_conditions_plus')) {
      $form['mautic_settings']['commerce_conditions'] = [
        '#type' => 'commerce_conditions_table',
        '#title' => $this->t('Applies to'),
        '#parent_entity_type' => 'mautic_commerce_condition',
        '#entity_types' => ['commerce_order'],
        '#default_value' => is_array($this->entity->getSettings('commerce_conditions')) ? $this->entity->getSettings('commerce_conditions') : [],
      ];
    }
    else {
      $form['mautic_settings']['commerce_conditions'] = [
        '#type' => 'commerce_conditions',
        '#title' => $this->t('Applies to'),
        '#parent_entity_type' => 'mautic_commerce_condition',
        '#entity_types' => ['commerce_order'],
        '#default_value' => is_array($this->entity->getSettings('commerce_conditions')) ? $this->entity->getSettings('commerce_conditions') : [],
        '#states' => [
          'visible' => [
            ':input[id="edit-mautic-segment"]' => ['!value' => ''],
          ],
        ],
      ];
    }

    $form['mautic_settings']['operator'] = [
      '#title' => $this->t('Condition operator'),
      '#type' => 'radios',
      '#options' => [
        'AND' => $this->t('All conditions must pass'),
        'OR' => $this->t('Only one condition must pass'),
      ],
      '#default_value' => $this->entity->getSettings('operator') ?? 'OR',
    ];

    $form['mautic_settings']['exclude_from_marketing_terms'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Exclude from marketing terms'),
      '#default_value' => $this->entity->getSettings('exclude_from_marketing_terms') ?? FALSE,
    ];

    $form['status'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enabled'),
      '#default_value' => $this->entity->status(),
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {

    foreach ($form['mautic_settings'] as $key => $value) {
      if (substr($key, 0, 1) != '#' && array_key_exists($key, $form_state->getValues())) {
        $this->entity->setSetting($key, $form_state->getValue($key));
      }
    }
    $result = parent::save($form, $form_state);
    $message_args = ['%label' => $this->entity->label()];
    $message = $result == SAVED_NEW
      ? $this->t('Created new mautic commerce condition %label.', $message_args)
      : $this->t('Updated mautic commerce condition %label.', $message_args);
    $this->messenger()->addStatus($message);
    $form_state->setRedirectUrl($this->entity->toUrl('collection'));
    return $result;
  }

  /**
   * Ajax callback.
   */
  public static function ajaxRefresh(array $form, FormStateInterface $form_state) {
    return $form;
  }

}
